import scipy as sp
import string
import scipy
import scipy.stats
import math
import sys
import cStringIO
# Copyright 2011, Robert G. Petry
# This code is licensed under the GPL version 3 or later.
#
# Go to the end of the code to see how to use this script.
#
#The following function deals with a few rounding errors that can
#show up in the tables due to the fact that python and scipy.stats
#use the native floats which can produce things like:
#scipy.stats.binom.cdf(1,2,.05)=0.99749999999999994
#for the binomial cumulative probability for n=2, k=1, p=.05
#This should be .9975 exactly which would then round to .998 when
#displaying three decimals but unfortunately rounds to .997.
#To fix this note that printing this number or calling str() actually
#produces .9975, i.e. the algorithm recognizes that this is a rounding
#issue and fixes it.  If the string .9975 is converted to a float and
#then printed it to three decimals it will now correctly produces .998.
#All that blather explains this function:
def fixRound(x):
    return float(str(x))
#def factorial(n):
#    return round(sp.factorial(n).tolist())
#def permutation(n,r):
#    return factorial(n)/factorial(n-r)
#def combination(n,r):
#    return permutation(n,r)/factorial(r)
#def binomial(n,p,x):
#    return combination(n,x)*p**x*(1-p)**(n-x)
def binomial_table(n,p=[.01,.05,.1,.2,.3,.4,.5,.6,.7,.8,.9,.95,.99]):
    print r"\begin{center}\resizebox{\textwidth}{!}{%"
    print r"\mbox{\begin{tabular}{r"+(len(p))*"c"+r"}"
    print r"\multicolumn{"+`len(p)`+r"}{l}{\ns="+`n`+r"}\\ \toprule"
    print r" & \multicolumn{"+`len(p)`+r"}{c}{\pprop}\\ \cmidrule{2-"+`len(p)+1`+r"}"
    print r"$\X$"+string.join([" & %7.2f"%p_tmp for p_tmp in p])+r"\\ \midrule"
    for x in range(n+1):
        if x!=n:
            endline=r"\\"
        else:
            endline=r"\\ \bottomrule"
        print "%4.0f  "%x+string.join([" & "+("%3.3f"% fixRound(scipy.stats.binom.pmf(x,n,p_tmp))) for p_tmp in p])+endline
    print r"\end{tabular}}}\end{center}"
def binomial_sheets(table_list):
    print r"\section*{Binomial Distribution}"
    print r"\label{page:binom}"
    for n in table_list:
        binomial_table(n)
    print r"\clearpage"
def cumulative_binomial_table(n,p=[.01,.05,.1,.2,.3,.4,.5,.6,.7,.8,.9,.95,.99]):
    print r"\begin{center}\resizebox{\textwidth}{!}{%"
    print r"\mbox{\begin{tabular}{r"+(len(p))*"c"+r"}"
    print r"\multicolumn{"+`len(p)`+r"}{l}{\ns="+`n`+r"}\\ \toprule"
    print r" & \multicolumn{"+`len(p)`+r"}{c}{\pprop}\\ \cmidrule{2-"+`len(p)+1`+r"}"
    print r"$\K$"+string.join([" & %7.2f"%p_tmp for p_tmp in p])+r"\\ \midrule"
    for k in range(n+1):
        if k!=n:
            endline=r"\\"
        else:
            endline=r"\\ \bottomrule"
        print "%4.0f  "%k+string.join([" & "+("%3.3f"% fixRound(scipy.stats.binom.cdf(k,n,p_tmp))) for p_tmp in p])+endline
    print r"\end{tabular}}}\end{center}"
def cumulative_binomial_sheets(table_list):
    print r"\section*{Cumulative Binomial Probabilities, $\bm{P(\X\le\K)}$}"
    print r"\label{page:cbinom}"
    print r"\vspace{1em}"
    print r"\begin{center}"
    print r"""\begin{asy}
import graph;
import stats;
// The zerof and invisible draw below are to make the right axis be
// properly aligned.
size(300,75,IgnoreAspect);
real[] bins={-.5,.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5};
real[] mids={-1,0,1,2,3,4,5,6,7,8};
real[] myticks={0,1,3};
//Do binomial for n=7, p=.35
real[] count={0.049,0.185,0.298,0.268,0.144,0.047,0.008,0.001};
real[] countwzeros={0,0.049,0.185,0.298,0.268,0.144,0.047,0.008,0.001,0};
real zerof(real x) {return 0.0;}
ylimits(0,.3);
fill((-.5,0)--(.5,0)--(.5,.049)--(-.5,.049)--cycle, grey);
fill((.5,0)--(1.5,0)--(1.5,.185)--(.5,.185)--cycle, grey);
fill((1.5,0)--(2.5,0)--(2.5,.298)--(1.5,.298)--cycle, grey);
fill((2.5,0)--(3.5,0)--(3.5,.268)--(2.5,.268)--cycle, grey);
histogram(bins, count, bars=true);
xaxis("",xmin=min(bins)-(bins[1]-bins[0]),xmax=max(bins)+(bins[1]-bins[0]),Bottom,LeftTicks(Ticks=myticks,OmitFormat(0,1,3)), above=true);
yaxis("",Left);
label("$0$",(0,0),S);
label("$1$",(1,0),S);
label("$\cdots$",(2,-.02),S);
label("$\K$",(3,0),S);
label("$\X$",(8.3,0),S);
label("$P(\X)$",(-1.5,.3),W);
label("$P(\X\le\K)=\mbox{Area}=P(0)+P(1)+\ldots + P(\K)$",(4,.3),E);
draw((4,.3){left}..{left}(1.7,.16),Arrow);
\end{asy}"""
    print r"\end{center}"
    for n in table_list:
        cumulative_binomial_table(n)
    print r"\clearpage"
def poisson_table(mu=scipy.arange(.1,1.1,.1)):
    print r"\begin{center}"
    print r"\mbox{\begin{tabular}{r"+(len(mu))*"c"+r"} \toprule"
    print r" & \multicolumn{"+`len(mu)`+r"}{c}{$\mu$}\\ \cmidrule{2-"+`len(mu)+1`+r"}"
    print r"$\X$"+string.join([" & %7.1f"%mu_tmp for mu_tmp in mu])+r"\\ \midrule"
    x=-1
    print_str=""
    while print_str.count("0.000") !=len(mu):
        x=x+1
        print_str="%4.0f  "%x+string.join([" & "+("%3.3f"% fixRound(scipy.stats.poisson.pmf(x,mu_tmp))) for mu_tmp in mu])+r"\\"
        print print_str
    print r"\bottomrule"
    print r"\end{tabular}}\end{center}"
def poisson_sheets():
    print r"\section*{Poisson Distribution}"
    print r"\label{page:poisson}"
    poisson_table((scipy.arange(.1,1.1,.1)).tolist()+[1.5])
    poisson_table(scipy.arange(2.0,7.5,.5))
    poisson_table((scipy.arange(7.5,10.5,.5)).tolist()+[12.0,15.0,20.0])
    print r"\clearpage"
def cumulative_poisson_table(mu=scipy.arange(.1,1.1,.1)):
    print r"\begin{center}"
    print r"\mbox{\begin{tabular}{r"+(len(mu))*"c"+r"} \toprule"
    print r" & \multicolumn{"+`len(mu)`+r"}{c}{$\mu$}\\ \cmidrule{2-"+`len(mu)+1`+r"}"
    print r"$\K$"+string.join([" & %7.1f"%mu_tmp for mu_tmp in mu])+r"\\ \midrule"
    k=-1
    print_str=""
    while print_str.count("1.000") !=len(mu):
        k=k+1
        print_str="%4.0f  "%k+string.join([" & "+("%3.3f"% fixRound(scipy.stats.poisson.cdf(k,mu_tmp))) for mu_tmp in mu])+r"\\"
        print print_str
    print r"\bottomrule"
    print r"\end{tabular}}\end{center}"
def cumulative_poisson_sheets():
    print r"\section*{Cumulative Poisson Probabilities, $\bm{P(\X\le\K)}$}"
    print r"\label{page:cpoisson}"
    print r"\vspace{1em}"
    print r"\begin{center}"
    print r"""\begin{asy}
import graph;
import stats;
// The zerof and invisible draw below are to make the right axis be
// properly aligned.
size(300,75,IgnoreAspect);
real[] bins={-.5,.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5};
real[] mids={-1,0,1,2,3,4,5,6,7,8};
real[] myticks={0,1,3};
//Do poisson for mu=1.5
real[] count={0.223,0.335,0.251,0.126,0.047,0.014,0.004,0.001};
real[] countwzeros={0,0.223,0.335,0.251,0.126,0.047,0.014,0.004,0.001,0};
real zerof(real x) {return 0.0;}
ylimits(0,.35);
fill((-.5,0)--(.5,0)--(.5,.223)--(-.5,.223)--cycle, grey);
fill((.5,0)--(1.5,0)--(1.5,.335)--(.5,.335)--cycle, grey);
fill((1.5,0)--(2.5,0)--(2.5,.251)--(1.5,.251)--cycle, grey);
fill((2.5,0)--(3.5,0)--(3.5,.126)--(2.5,.126)--cycle, grey);
histogram(bins, count, bars=true);
xaxis("",xmin=min(bins)-(bins[1]-bins[0]),xmax=max(bins)+(bins[1]-bins[0]),Bottom,LeftTicks(Ticks=myticks,OmitFormat(0,1,3)), above=true);
yaxis("",Left);
label("$0$",(0,0),S);
label("$1$",(1,0),S);
label("$\cdots$",(2,-.02),S);
label("$\K$",(3,0),S);
label("$\X$",(8.3,0),S);
label("$P(\X)$",(-1.5,.35),W);
label("$P(\X\le\K)=\mbox{Area}=P(0)+P(1)+\ldots + P(\K)$",(4,.3),E);
draw((4,.3){left}..{left}(1.7,.16),Arrow);
\end{asy}"""
    print r"\end{center}"
    cumulative_poisson_table((scipy.arange(.1,1.1,.1)).tolist()+[1.5])
    cumulative_poisson_table(scipy.arange(2.0,7.5,.5))
    cumulative_poisson_table((scipy.arange(7.5,10.5,.5)).tolist()+[12.0,15.0,20.0])
    print r"\clearpage"
# For normal distribution from zero to z area we use:
# scipy.stats.norm.cdf(x)-.5
def cumulative_normal_table():
    print r"\mbox{\begin{tabular}{cccccccccccc}\toprule"
    print r"$\Z$"+string.join([" & ~%4.2f"%decimal for decimal in scipy.arange(0,.1,.01)])+r"\\ \midrule"
    # The extra decimal keeps 0.0 slightly negative which ensures -0.0 prints. 
    for z in scipy.arange(-3.40000000001,0.05,.1):
        if z!=0.0:
            endline=r"\\"
        else:
            endline=r"\\ \bottomrule"
        print string.join([r"%2.1f"%z]+[" & "+("%6.4f"%(fixRound(scipy.stats.norm.cdf(z-decimal)))) for decimal in scipy.arange(0,.1,.01)])+endline
        if int(z*10)%5==0 and z!=-3.40000000001:
            print string.join([" & " for decimal in scipy.arange(0,.1,.01)])+endline
    print r"\end{tabular}}"
    print r"\clearpage~\vfill"
    print r"\mbox{\begin{tabular}{cccccccccccc}\toprule"
    print r"$\Z$"+string.join([" & ~%4.2f"%decimal for decimal in scipy.arange(0,.1,.01)])+r"\\ \midrule"
    for z in scipy.arange(0,3.5,.1):
        if z!=3.4:
            endline=r"\\"
        else:
            endline=r"\\ \bottomrule"
        print string.join([r"%2.1f"%z]+[" & "+("%6.4f"%(fixRound(scipy.stats.norm.cdf(z+decimal)))) for decimal in scipy.arange(0,.1,.01)])+endline
        if int(z*10+1)%5==0:
            print string.join([" & " for decimal in scipy.arange(0,.1,.01)])+endline
    print r"\end{tabular}}"

def cumulative_normal_sheet():
    print r"""\section*{Normal Curve Areas}
\label{page:cnormal}
\begin{center}
\begin{asy}
import graph;
import stats;
//size(300,100,IgnoreAspect);
size(200,60,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,-3.0,1.23,operator --)--(1.23,0)--(-3,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
labelx("$\Z$", (3,0),E);
label("1.23",(1.23,0),S);
label("0",(0,0),S);
label("$P(\Z<1.23)=\mbox{Area}=0.8907$",(1.1,1.5*normalf(1.1)),E);
draw((1.1,1.5*normalf(1.1)){left}..{left}(.5,.7*normalf(1)),Arrow);
xequals(x=0, ymin=0, ymax=normalf(0), dashed);
\end{asy}
\end{center}
\begin{center}
"""
    cumulative_normal_table()
    print r"""\vspace{-1em}
\end{center}
\clearpage"""
# For normal distribution from zero to z area we use:
# scipy.stats.norm.cdf(x)-.5
def normal_zero_to_z_table():
    print r"\mbox{\begin{tabular}{cccccccccccc}\toprule"
    print r"$\Z$"+string.join([" & ~%4.2f"%decimal for decimal in scipy.arange(0,.1,.01)])+r"\\ \midrule"
    for z in scipy.arange(0,3.5,.1):
        if z!=3.4:
            endline=r"\\"
        else:
            endline=r"\\ \bottomrule"
        print string.join([r"%2.1f"%z]+[" & "+("%6.4f"%(fixRound(scipy.stats.norm.cdf(z+decimal)-.5))) for decimal in scipy.arange(0,.1,.01)])+endline
        if int(z*10+1)%5==0:
            print string.join([" & " for decimal in scipy.arange(0,.1,.01)])+endline
    print r"\end{tabular}}"
def normal_zero_to_z_sheet():
    print r"""\section*{Normal Curve Areas}
\label{page:normal}
\begin{center}
\begin{asy}
import graph;
import stats;
//size(300,100,IgnoreAspect);
size(200,60,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,0,1.23,operator --)--(1.23,0)--(0,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
labelx("$\Z$", (3,0),E);
label("$1.23$",(1.23,0),S);
label("$0$",(0,0),S);
label("$P(0<\Z<1.23)=\mbox{Area}=0.3907$",(1.1,1.5*normalf(1.1)),E);
draw((1.1,1.5*normalf(1.1)){left}..{left}(.5,.7*normalf(1)),Arrow);
\end{asy}
\end{center}
\vfill
\begin{center}
"""
    normal_zero_to_z_table()
    print r"""\end{center}
\clearpage"""
#
# For Student's t Distribution use:
# scipy.stats.t.isf(alpha,df) where alpha is the level of significance for a one-tailed test
# and df is the number of degrees of freedom.
def t_table_header():
    print r"\resizebox{!}{3.1in}{\begin{tabular}{r|r|r|r|r|r|r}\hline"
    print r"\multirow{6}{2ex}[-2em]{\df} & \multicolumn{6}{c}{\raisebox{-.5em}{Confidence Coefficient, \conf}}\\ \cline{2-7}"
    print r"\rule{0em}{1.3em}& 0.80 & 0.90 & 0.95 & 0.98 & 0.99 & 0.999 \\ \cline{2-7}"
    print r"\rule{0em}{1.3em}& \multicolumn{6}{c}{Level of Significance for One-Tailed Test, $\sig$}\\ \cline{2-7}"
    print r"\rule{0em}{1.3em}& 0.100 & 0.050 & 0.025 & 0.010 & 0.005 & 0.0005 \\ \cline{2-7}"
    print r"\rule{0em}{1.3em}& \multicolumn{6}{c}{Level of Significance for Two-Tailed Test, $\sig$}\\ \cline{2-7}"
    print r"\rule{0em}{1.3em}& 0.20 & 0.10 & 0.05 & 0.02 & 0.01 & 0.001\\ \hline"
    print r" & & & & & & \\"

def t_table():
    print r"\begin{multicols}{2}%"
    t_table_header()
    dfrange=range(1,31,1)
    for df in dfrange:
        if df!=dfrange[-1]:
            endline=r"\\"
        else:
            endline=r""
        #print string.join([r"\rule{0em}{1.3em}"+`df`]+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
        print string.join([`df`]+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
        if df%5 == 0 and df != dfrange[-1]:
            print string.join([" & " for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
    print r"\end{tabular}}"
    t_table_header()
    dfrange=range(31,56,1)+[60,80,100,200,1000000]
    for df in dfrange:
        if df!=dfrange[-1]:
            endline=r"\\"
        else:
            endline=r""
        if df != dfrange[-1]:
            #print string.join([r"\rule{0em}{1.3em}"+`df`]+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
            print string.join([`df`]+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
            if df%5 == 0 and df < 60:
                print string.join([" & " for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
        else:
            #print string.join([r"\rule{0em}{1.3em}"+'$\infty$']+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
            print string.join(['$\infty$']+[" & "+("%5.3f"%(fixRound(scipy.stats.t.isf(alpha,df)))) for alpha in [0.100,0.050,0.025,0.010,0.005,0.0005]])+endline
    print r"\end{tabular}}"
    print r"\end{multicols}"

def t_sheet():
    print r"""\section*{Student's $\bm\T$ Distribution (Critical Values)}
\label{page:t}
\begin{center}
\newbox\mybox
\setbox\mybox=\hbox{%
\begin{asy}
import graph;
import stats;
size(115,80,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,-1.23,1.23,operator --)--(1.23,0)--(-1.23,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
label("$\T$",(1.23,0),S);
label("$-\T$",(-1.35,0),S);
label("Confidence Interval",(0,-.25*normalf(0)),S);
label("\conf",(1.1,1.5*normalf(1.1)),E);
draw((1.1,1.5*normalf(1.1)){left}..{left}(.5,.7*normalf(1)),Arrow);
\end{asy}
\hspace{2em}
%Left Tailed Diagram
\begin{asy}
import graph;
import stats;
size(115,80,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,-3.00,-1.23,operator --)--(-1.23,0)--(-3.00,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
label("$-\T$",(-1.35,0),S);
label("Left-tailed Test",(0,-.25*normalf(0)),S);
label("$\sig$",(-2.5,6*normalf(-2.5)),W);
draw((-2.5,6*normalf(-2.5)){right}..{right}(-1.5,.3*normalf(-1.5)),Arrow);
\end{asy}
\hspace{2em}
%Right Tailed Diagram
\begin{asy}
import graph;
import stats;
size(115,80,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,3.00,1.23,operator --)--(1.23,0)--(3.00,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
label("$\T$",(1.23,0),S);
label("Right-tailed Test",(0,-.25*normalf(0)),S);
label("$\sig$",(2.5,6*normalf(2.5)),E);
draw((2.5,6*normalf(2.5)){left}..{left}(1.5,.3*normalf(1.5)),Arrow);
\end{asy}
\hspace{2em}
%Two Tailed Diagram
\begin{asy}
import graph;
import stats;
size(115,80,IgnoreAspect);
real zerof(real x) {return 0.0;}
real normalf(real x) {return 1/(sqrt(2pi)*1)*exp(-.5((x-0)/1)^2);}
path g=graph(normalf,3.00,1.23,operator --)--(1.23,0)--(3.00,0)--cycle;
filldraw(g,grey);
path g=graph(normalf,-3.00,-1.23,operator --)--(-1.23,0)--(-3.00,0)--cycle;
filldraw(g,grey);
draw(graph(normalf,-3,3),black+1.5);
ylimits(0,1.2*normalf(0));
xaxis(Bottom,LeftTicks(Step=1,OmitFormat(-2,-1,0,1,2),OmitTick(-3,3),Size=3),above=true);
label("$\T$",(1.23,0),S);
label("$-\T$",(-1.35,0),S);
label("Two-tailed Test",(0,-.25*normalf(0)),S);
label("$\frac{\sig}{2}$",(-2.5,6*normalf(-2.5)),W);
draw((-2.5,6*normalf(-2.5)){right}..{right}(-1.5,.3*normalf(-1.5)),Arrow);
label("$\frac{\sig}{2}$",(2.5,6*normalf(2.5)),E);
draw((2.5,6*normalf(2.5)){left}..{left}(1.5,.3*normalf(1.5)),Arrow);
\end{asy}
}
\resizebox{\textwidth}{!}{\usebox\mybox}
\end{center}
\begin{center}
"""
    t_table()
    print r"""\end{center}
\clearpage
"""
def trimdigits(x):
    """This function takes a real number and produces a five digit
output (counting a decimal place if there is room to put something
after it, otherwise padding with a tilde ~, a fixed LaTeX space on the
left.  eg. 2039.3838 would give ~2039.  Obviously this limits the
magnitude to five digits maximum, but for our F table we don't go as
high as 99999 and nothing is smaller than 1.  Note the numbers are properly rounded."""
    powx=math.trunc(math.log(x,10))
    if powx==3 or powx==4:
        dec=0
    else:
        dec=3-powx
    output = cStringIO.StringIO()
    formatstr="{0:."+str(dec)+"f}"
    #Note the float(str()) stuff deals with numbers like
    #19999.499999997 which then get converted to 19999.5 properly by str
    #and then will properly round to 20000.
    output.write(formatstr.format(float(str(x))))
    xstr= output.getvalue()
    output.close()
    if powx==3:
        xstr="~"+xstr
    return xstr

# The following large degree of freedom was chosen as infinity.  This
# was the smallest power of 10 that gave all F values equal to 1.000
# for df1=infty and df2=infty.  The next largest power of 10 differed by
# a single smallest digit in one of the infty entries. However by 
#     100000000000 there were many more differences occurring and numerical
# errors were clearly appearing.  The infinity entries in other tables go
# only to two decimals and my table agrees with those except for
# df1=infty, df2=1, alpha=0.005 where I have 25464 and texts have 25465.
infty=1000000000

def f_table(df1range=range(1,5),
            df2range=range(1,31)+[40, 50, 60, 70, 80, infty],
            alphalist=[.1,.05,.025,.01,.005]):
    """Prints LaTeX code sufficient to generate a single page table of the F
    distribution.  The defaults are suitable to make the first page of such a
    table that will fit on letter paper (in landscape mode).  To print a
    multiple page table which goes beyond just df_1 equal 4 see fmultipage that
    calls this function, or just pass a new list of df1 values in the df1range
    argument.  Similarly the df2range and alphalist  list arguments can be
    passed to modify that behaviour as well.  The calculation is done by
    calling scipy.stats.f.isf(alpha,df1,df2)"""
    alphaliststr=string.join(["{0:.3f}".format(alpha) for alpha in alphalist],r"~\,")
    df1perpage=len(df1range)
    df1str={}
    for df1 in df1range:
        if df1==infty:
            df1str[df1]=r"$\infty$"
        else:
            df1str[df1]=str(df1)
    print(r"{\tt\small")
    print(r"\begin{tabular}{cr"+df1perpage*"c"+r"}")
    print(r"\multirow{"+str(len(df2range)+10)+r"}{2ex}{$\df_2$}& & \multicolumn{"+str(df1perpage)+r"}{c}{$\df_1$}\\\cmidrule(lr){3-"+str(df1perpage+2)+r"}")
    print(r"&"+string.join([" & "+df1str[df1] for df1 in df1range],"")+r" \\ "+string.join([r"\cmidrule(lr){"+str(i)+"-"+str(i)+r"}" for i in range(3,3+df1perpage)],""))
    df2count=0
    for df2 in df2range:
        df2count +=1
        if df2 != infty:
            sys.stdout.write(' & \multicolumn{1}{|r|}{'+str(df2)+ '}& ')
        else:
            sys.stdout.write(' & \multicolumn{1}{|r|}{$\infty$}& ')
        for df1 in df1range:
            for alpha in alphalist:
                sys.stdout.write(trimdigits(scipy.stats.f.isf(alpha,df1,df2)))
                if alpha!=.005:
                    sys.stdout.write('~\,')
            if df1 != df1range[-1]:
                sys.stdout.write(' & ')
            else:
                if df2count%5==0 and df2!=infty:
                    print(r' \\[.4em]')
                else:
                    print(r' \\')
    print(r"\cmidrule(lr){3-"+str(2+df1perpage)+"}")
    print("& & "+(df1perpage-1)*(alphaliststr+" & ")+alphaliststr
          +r"\\ \cmidrule(lr){3-"+str(2+df1perpage)+"}")
    print(r"& & \multicolumn{"+str(df1perpage)+r"}{c}{$\sig$}")
    print(r"\end{tabular}}")     

def f_sheets(df1range=range(1,31)+[40, 60, 80, 100, infty],
             df2range=range(1,31)+[40, 60, 80, 100, infty],
             alphalist=[.1,.05,.025,.01,.005], df1perpage=5):
    """Prints a multiple page set of tables in LaTeX form for the F
    distribution.  See fpage for a single page option.  That function is
    called from this one.  The parameters (df1range, df2range and alphalist)
    are the same. The last parameter, df1perpage, says how many df1 columns
    you want on a page.  The default is 4 which is suitable for a
    landscape mode table.  The default values will print a reasonable table."""
    print(r"""\section*{The $\bm\F$ Distribution (Critical Values)}
\label{page:f}
\vfill
\begin{center}
\begin{sideways}
\hspace{.5in}
\begin{asy}
import graph;
size(125,60,IgnoreAspect);
// Degrees of freedom for our F distribution probability density function.
real df1;
real df2;
df1=6.0;
df2=6.0;
// This coefficient is calculated once and for all for our F distribution.
real Fcoeff;
Fcoeff=gamma((df1+df2)/2)*(df1/df2)**(df1/2)/(gamma(df1/2)*gamma(df2/2));
real zerof(real x) {return 0.0;}
// We calculate the F pdf as asymptote doesn't have it.
real Fpdf(real x) {return Fcoeff*x**(df1/2-1)/(1+df1*x/df2)**((df1+df2)/2);}
path g=graph(Fpdf,2.5,3.5,operator --)--(3.5,0)--(2.5,0)--cycle;
filldraw(g,grey);
draw(graph(Fpdf,0,3.5),black+1.5);
ylimits(0,1.5*Fpdf(1));
xaxis(Bottom,above=true);
yaxis(Left);
label("$\F$", (2.5,0),S);
label("0",(0,0),S);
label("$\sig$",(3.2,2*Fpdf(2.5)),E);
draw((3.25,2*Fpdf(2.5)){left}..{left}(2.7,.5*Fpdf(2.7)),Arrow);
\end{asy}
\end{sideways}""")
    df1remaining=True
    firstpage=True
    while(df1remaining):
        df1subrange=[df1range.pop(0) for i in range(min(df1perpage,len(df1range)))]
        if len(df1subrange)>0:
            if not firstpage:
                print(r"~\vfill\begin{center}")
            print(r"\begin{sideways}\resizebox{!}{2.4in}%")
            f_table(df1subrange,df2range,alphalist)
            print(r"\end{sideways}")
            print(r"\end{center}")
            print(r"\clearpage")
        else:
            df1remaining=False
        firstpage=False
#The following preamble and postscript are required to make a freestanding
#compilable formula sheet.  Notice that you have to define several variables
#that affect the appearance of your variables in the doc.  These are contained
#in statssymbols.sty which can be modified to suit your needs.  To create a
#table that goes inline with your document you don't need the preamble, etc.
#but you still need to define the variables by including your statssymbols.sty
def preamble():
    print r"""\documentclass{article}
\usepackage{booktabs}
\usepackage[outer=3.5cm,top=2.75cm,inner=2.5cm,bottom=2.75cm]{geometry}
\usepackage[inline]{asymptote}
\usepackage{multirow}
\usepackage{multicol}
\usepackage{bm}
\usepackage{rotating}
\usepackage{statssymbols}
\usepackage{nopageno}%Comment out if you want page numbers.
\begin{document}
"""
def postscript():
    print r"""\end{document}"""

#Uncomment the types of tables you want.  If you want a stand-alone
#set of tables include the preamble() and postscript() functions and run
## python stattables.py > stattables.tex
## pdflatex stattables.tex
## asy stattables
## pdflatex stattables.tex
#
#If you want to create an inline document to input into your current
#document comment out preamble() and postscript() and just do
## python stattables.py > stattables_inline.tex
#Then input stattables_inline.tex into your document.  The tables require
#asymptote to be run to generate the figures as shown above.
#
#preamble()
##binomial_sheets(range(1,21)+[25])
cumulative_binomial_sheets(range(1,21)+[25])
##poisson_sheets()
##cumulative_poisson_sheets()
##normal_zero_to_z_sheet()
cumulative_normal_sheet()
t_sheet()
f_sheets()
#postscript()
